using System;
using System.IO;
using System.Xml;
using System.Collections.Generic;
using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;
using System.Xml.XPath;


namespace PrototypAssertion
{
    class TestAssertion : SecurityPolicyAssertion
    {
        XmlDocument mapping = new XmlDocument();
        XmlDocument repository = new XmlDocument();

        public override void ReadXml(XmlReader reader, IDictionary<string, Type> extensions)
        {
            if (reader == null)
                throw new ArgumentNullException("reader");
            if (extensions == null)
                throw new ArgumentNullException("extensions");

            string extensionName = null;
            foreach (KeyValuePair<string, Type> pair in extensions)
            {
                if (pair.Value == typeof(PrototypAssertion.TestAssertion))
                {
                    extensionName = pair.Key;
                    break;
                }
            }

            reader.ReadStartElement(extensionName);

            // Read mappings section
            if (reader.MoveToContent() == XmlNodeType.Element)
            {
                if (reader.LocalName != "mapping")
                    throw new Exception("The mapping element must be the first element");

                XmlReader mappingReader = reader.ReadSubtree();
                mapping.Load(mappingReader);
                mappingReader.Close();

                // Closing mapping
                reader.ReadEndElement();
            }

            // Read repository section
            if (reader.MoveToContent() == XmlNodeType.Element)
            {
                if (reader.LocalName != "repository")
                    throw new Exception("The repository element must be the second element");

                XmlReader repositoryReader = reader.ReadSubtree();
                repository.Load(repositoryReader);
                repositoryReader.Close();

                // Closing repository
                reader.ReadEndElement();
            }

            // Closing proto
            reader.ReadEndElement();


        }

        public override SoapFilter CreateClientInputFilter(FilterCreationContext context)
        {
            return null;
        }

        public override SoapFilter CreateClientOutputFilter(FilterCreationContext context)
        {
            return null;
        }

        public override SoapFilter CreateServiceInputFilter(FilterCreationContext context)
        {
            return new TestFilter(mapping, repository);
        }

        public override SoapFilter CreateServiceOutputFilter(FilterCreationContext context)
        {
            return null;
        }
    }
}
