using System;
using System.IO;
using System.Xml;
using System.Collections.Generic;
using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;
using System.Xml.XPath;
using System.Web.Services.Protocols;


namespace InputValidator
{
    // MessageInterceptor is part of the Validation component.
    // It inherits from SOAP Filter.
    // It communicates with WSE and the Validator.
    // Its task is to intercept incoming SOAP envelopes and ask the Validator
    // to validate them. If the messages are found valid, they are forwarded
    // to the web service, if not a SOAP Fault is returned to the sender.
    class MessageInterceptor : SoapFilter
    {

        Validator validator;

        public MessageInterceptor(XmlDocument mapping,
          XmlDocument repository)
        {
            validator = new Validator(
              new PolicyMappingManager(mapping),
              new RepositorySchemaProvider(repository),
              new WsdlSchemaProvider(mapping));
        }

        // The message that intercepts the SOAP Envelope
        public override SoapFilterResult
          ProcessMessage(SoapEnvelope envelope)
        {
            if (validator.IsValid(envelope))
            {
                // If the message is valid, forward it
                return SoapFilterResult.Continue;
            }
            else
            {
                // When a SoapException is thrown and not caught, the server returns a SOAP fault to the sender
                throw new SoapException("Validation failed", SoapException.ClientFaultCode);
            }
        }
    }
}
