using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace InputValidator
{
    // PolicyFileHelper is part of the Verification component.
    // It communicates with the ValidationAssertion.
    // Its task is to encapsulate the logic of extracting mapping
    // and schema information from the WSE policy file
    public class PolicyFileHelper
    {

        private XmlReader reader = null;
        private ValidationLog logger;

        public PolicyFileHelper(XmlReader r)
        {
            reader = r;
            logger = 
              new ValidationLog("Validator", "InputValidator");
        }

        // Returns the mapping
        public System.Xml.XmlDocument GetMapping()
        {
            XmlDocument mapping = new XmlDocument();

            // Read mappings section
            if (reader.MoveToContent() ==
              XmlNodeType.Element)
            {
                if (reader.LocalName != "mapping")
                {
                    logger.Write("The mapping element must be the first element.");
                    throw new Exception("The mapping element must be the first element");
                }

                XmlReader mappingReader =
                  reader.ReadSubtree();
                mapping.Load(mappingReader);
                mappingReader.Close();

                // Closing mapping
                reader.ReadEndElement();
            }

            return mapping;
        }

        // Returns the repository schema information
        public XmlDocument getRepository()
        {
            XmlDocument repository = new XmlDocument();

            // Read repository section
            if (reader.MoveToContent() == 
              XmlNodeType.Element)
            {
                if (reader.LocalName != "repository")
                {
                    logger.Write("The repository element must be the second element.");
                    throw new Exception("The repository element must be the second element.");
                }

                XmlReader repositoryReader = 
                  reader.ReadSubtree();
                repository.Load(repositoryReader);
                repositoryReader.Close();

                // Closing repository
                reader.ReadEndElement();
            }

            return repository;

        }
    }
}
