using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace InputValidator
{
  // PolicyMappingManager is part of the Policy component.
  // It communicates with the Validator and ValidationLog.
  // Its task is to return the mapping (policy) for a given
  // web service
  public class PolicyMappingManager
  {
    private XmlDocument mapping;
    private XmlNamespaceManager xmlnsm;
    private ValidationLog logger;

    public PolicyMappingManager(XmlDocument mapping)
    {
      this.mapping = mapping;
      xmlnsm = new XmlNamespaceManager(mapping.NameTable);
      xmlnsm.
        AddNamespace("item", "http://www.item.ntnu.no/ws");
      logger = new ValidationLog(
        "PolicyMappingManager",
        "InputValidator");
    }

    // Returns the mapping for a given web service
    public XmlDocument GetMapping(
      string uri,
      string port,
      string namesp)
    {
      XmlElement portNode = (XmlElement)mapping.SelectSingleNode("/item:mapping/item:webservice/item:ports/item:port[./@name='" + port + "' and ./@namespace='" + namesp + "' and ./../../item:uris/item:uri[.='" + uri + "']]", xmlnsm);

      if (portNode == null)
      {
        logger.Write("Mapping not defined for the requested web service.\n uri: " + uri + "\n port: " + port + " \n namespace: " + namesp);
        throw new Exception("Port or URI not matching");
      }

      XmlDocument d = new XmlDocument();
      portNode = (XmlElement)d.ImportNode(portNode, true);
      d.AppendChild(portNode);
      return d;
    }
  }
}
