using System;
using System.IO;
using System.Xml;
using System.Collections.Generic;
using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;
using System.Xml.XPath;

namespace InputValidator
{
  // ValidationAssertion is part of the Verification component.
  // It inherits from SecurityPolicyAssertion
  // It communicates with WSE and PolicyFileHelper.
  // Its task is to get the WSE policy file from WSE, and to
  // return an instance of MessageInterceptor to WSE.
  public class ValidationAssertion : SecurityPolicyAssertion
  {
    private XmlDocument mapping = new XmlDocument();
    private XmlDocument repository = new XmlDocument();

    // Called by WSE. The XmlReader is a handle to the WSE
    // policy file
    public override void ReadXml(XmlReader reader,
      IDictionary<string,Type> extensions)
    {
      if (reader == null)
        throw new ArgumentNullException("reader");
      if (extensions == null)
        throw new ArgumentNullException("extensions");

      string extensionName = null;
      foreach (KeyValuePair<string, Type> pair in extensions)
      {
        if (pair.Value == 
          typeof(InputValidator.ValidationAssertion))
        {
          extensionName = pair.Key;
          break;
        }
      }

      PolicyFileHelper pfh = new PolicyFileHelper(reader);

      reader.ReadStartElement(extensionName);

      mapping = pfh.GetMapping();

      repository = pfh.getRepository();

      reader.ReadEndElement();


    }

    public override SoapFilter CreateClientInputFilter
      (FilterCreationContext context)
    {
      return null;
    }

    public override SoapFilter CreateClientOutputFilter
      (FilterCreationContext context)
    {
      return null;
    }

    // Returns an instance of MessageInterceptor to WSE
    // which will be applied to incoming messages on
    // the server side.
    public override SoapFilter CreateServiceInputFilter
      (FilterCreationContext context)
    {
      return new MessageInterceptor(mapping, repository);
    }

    public override SoapFilter CreateServiceOutputFilter
      (FilterCreationContext context)
    {
      return null;
    }
  }
}
